﻿--[[
	MrtWoo_AntiSpam: Detects and blocks russian commercial spam and flud
	Copyright (C) 2010 Pavel Dudkovsky (mrtime@era.by)

	This module based on SpamMeNot (filter.lua)
	Copyright (C) 2008 Robert Stiles (robs@codexsoftware.co.uk)

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
]]

MrtWoo_AntiSpamFilter = {}

MrtWoo_AntiSpamFilter.Words = {};

function MrtWoo_AntiSpamFilter:new(o)
	o = o or {}
	setmetatable(o, self)
	self.__index = self
	o.weights = self.Words;
	o.cachedRatings = {}
	return o
end


function MrtWoo_AntiSpamFilter:ReloadWords()
	self.weights = self.Words;
end;


function MrtWoo_AntiSpamFilter:RemoveHyperLinks(text)
	text = string.gsub(text, "|H.-|h(.-)|h", "%1")
	text = string.gsub(text, "|c%w%w%w%w%w%w%w%w(.-)|r", "%1")
	return text
end

-- Tests individual words and returns a summed spam rating.  Words
-- listed in wordsChecked are not checked again
function MrtWoo_AntiSpamFilter:TestWords(weights, wordsChecked, s)
	local w = ""
	local weight = 0
	-- Check individual words
	--for w in string.gmatch(s, "%w+") do
	for w in string.gmatch(s, "[1234567890abcdefghijklmnopqrstuvwxyzrабвгдеёжзийклмнопрстуфхцчшщъьыэюя]+") do
		if (not wordsChecked[w]) then
			if (weights[w]) then
				weight = weight + weights[w]
			end
			wordsChecked[w] = 1;
			-- MrtWoo:Print(w);
		end
	end
	return weight;
end

function MrtWoo_AntiSpamFilter:NumbersToLetters(s)
	s = string.gsub(s, "0" , "o")
	s = string.gsub(s, "1" , "l")
	s = string.gsub(s, "3" , "e")
	s = string.gsub(s, "4" , "a")
	s = string.gsub(s, "5" , "s")
	return s
end;

function MrtWoo_AntiSpamFilter:LettersToNumbers(s)
	s = string.gsub(s, "o" , "0")
	s = string.gsub(s, "l" , "1")
	s = string.gsub(s, "e" , "3")
	s = string.gsub(s, "a" , "4")
	s = string.gsub(s, "s" , "5")
	return s
end;

function MrtWoo_AntiSpamFilter:LatToRus(s)
	s = string.gsub(s, "e", "е");
	s = string.gsub(s, "t", "т");
	s = string.gsub(s, "o", "о");
	s = string.gsub(s, "p", "р");
	s = string.gsub(s, "a", "а");
	s = string.gsub(s, "h", "н");
	s = string.gsub(s, "k", "к");
	s = string.gsub(s, "x", "х");
	s = string.gsub(s, "c", "с");
	s = string.gsub(s, "b", "в");
	s = string.gsub(s, "m", "м");
	return s
end;

function MrtWoo_AntiSpamFilter:RusToLat(s)
	s = string.gsub(s, "е", "e");
	s = string.gsub(s, "т", "t");
	s = string.gsub(s, "о", "o");
	s = string.gsub(s, "р", "p");
	s = string.gsub(s, "а", "a");
	s = string.gsub(s, "н", "h");
	s = string.gsub(s, "к", "k");
	s = string.gsub(s, "х", "x");
	s = string.gsub(s, "с", "c");
	s = string.gsub(s, "в", "b");
	s = string.gsub(s, "м", "m");
	return s
end;

-- Searches for substring matches for words in the word list
-- Will not check for words listed in wordsFound
function MrtWoo_AntiSpamFilter:TestSubstringWords(weights, wordsFound, s)
	local word = ""
	local value = 0
	local weight = 0

	for word, value in pairs(weights) do
		if not wordsFound[word] then
			if (string.match(s, word)) then
				weight = weight + value
				wordsFound[word] = 1
			end
		end
	end
	return weight
end

-- Regular spaced word check.
function MrtWoo_AntiSpamFilter:SpacedWordCheck(weights, s)
	local weight = 0
	local wordsChecked = {}

	-- Check individual words
	weight = weight + self:TestWords(weights, wordsChecked, s)

	-- Remove double spacing and replace odd characters used for spaces with real ones
	-- and check again
	local spacestrip = "[^1234567890abcdefghijklmnopqrstuvwxyzr&ЂЈ$!.,%<>=-?абвгдеёжзийклмнопрстуфхцчшщъьыэюя]+"
	s = string.gsub(s, spacestrip, " ")
	weight = weight + self:TestWords(weights, wordsChecked, s)

	-- Change numbers commonly used as letters to their letter and check again
	s = self:NumbersToLetters(s)
	weight = weight + self:TestWords(weights, wordsChecked, s)

	-- and vice-versa
	s = self:LettersToNumbers(s)
	weight = weight + self:TestWords(weights, wordsChecked, s)

	-- and russian
	s = self:LatToRus(s)
	weight = weight + self:TestWords(weights, wordsChecked, s)

	--SpamMeNot:DebugMsg("SpaceWordCheck: " .. weight);

	return weight
end

-- Simply search for word matches anywhere in the text
function MrtWoo_AntiSpamFilter:SubstringCheck(weights, s)
	local word = ""
	local value = 0
	local weight = 0

	local wordsFound = {}

	weight = self:TestSubstringWords(weights, wordsFound, s)

	-- Revert numerics
	s = self:NumbersToLetters(s)
	weight = weight + self:TestSubstringWords(weights, wordsFound, s)

	-- and backwards
	s = self:LettersToNumbers(s)
	weight = weight + self:TestSubstringWords(weights, wordsFound, s)

	-- and russian
	s = self:LatToRus(s)
	weight = weight + self:TestSubstringWords(weights, wordsFound, s)

	--SpamMeNot:DebugMsg("SubstringCheck: " .. weight)
	return weight
end

-- The main spam rating formula.  Takes a string and returns a rating.  >= 100 is considered
-- to be spam.
function MrtWoo_AntiSpamFilter:RateMessage(weights, s)
	if type(weights) == "string" then
		s = weights
		weights = self.weights
	end

	--SpamMeNot:DebugMsg("RateMessage: " .. s)

	-- Strip out wow hyperlinks and colors
	s = self:RemoveHyperLinks(s)
	s = string.lower(s)

	if self.cachedRatings[s] then
		if self.cachedRatings[s][weights] then
			--SpamMeNot:DebugMsg("Returning cached value: " .. self.cachedRatings[s][weights])
			return self.cachedRatings[s][weights]
		end
	end

	local spacestrip = "[^1234567890abcdefghijklmnopqrstuvwxyzr&ЂЈ$!.,%<>=-?абвгдеёжзийклмнопрстуфхцчшщъьыэюя]+"
	local sCompact = string.gsub(s, spacestrip, "")

	local weight1 = self:SpacedWordCheck(weights, s)
	local weight2 = self:SubstringCheck(weights, s)
	local weight3 = self:SubstringCheck(weights, sCompact)

	local weight = weight1
	if weight2 > weight then
		weight = weight2
	end

	if weight3 > weight then
		weight = weight3
	end

	if not self.cachedRatings[s] then
		self.cachedRatings[s] = {}
	end

	self.cachedRatings[s][weights] = weight
	return weight
end

function MrtWoo_AntiSpamFilter:ContainsSkillLink(s)
	 --[[
		|Hspell:3127|h[Parry]|h
	]]

	local res = false
	if string.find(s, "|Hspell.-|h(.-)|h") then
		res = true
	end
	return res
end;

function MrtWoo_AntiSpamFilter:RateTradeMessage(s)

	local tradeSkillLinkWeight = 10
	local weight = self:RateMessage(s)

	if self:ContainsSkillLink(s) then
		weight = weight + tradeSkillLinkWeight
	end

	return weight
end
